/* $Id: hellosign.cpp,v 1.16 2011/02/24 13:19:12 rjs Exp $
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * hellosign sample: generate a document in memory with PDFlib, and sign it
 * with PLOP
 */

#include <iostream>

#include "pdflib.hpp"
#include "plop.hpp"

using namespace std;
using namespace pdflib;

int
main()
{
    int font;
    const char *contents = NULL;
    long size;
    PDFlib p;	

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    wstring out_filename(L"hellosign.pdf");

    static const wstring sign_opts = L"sign={"
	    L"appearance={fieldname=Signature1 } "
#if defined(WIN32) && defined(USE_MSCAPI)
            L"engine=mscapi "
            L"digitalid={certstore={store=My subject={DEMO PLOP user 2048}}} "
#else
            L"digitalid={filename=demo2048.p12} "
#endif
	    L"password={demo} "
	    L"reason={I agree to this document} "
	    L"contactinfo={tel (123) 456 789} "
	    L"location=Munich "
	    L"}";

    /* ------------- Step 1: generate the document in memory ------------ */

    try {
	/* open new PDF file in memory */

	p.begin_document(L"", L"");

	p.set_info(L"Creator", L"hellosign.c");
	p.set_info(L"Author", L"Rainer Pl\u00f6ckl");
	p.set_info(L"Title", L"Hello, world/signed (C)!");

	font = p.load_font(L"Helvetica-Bold", L"host", L"");

	p.begin_page_ext(a4_width, a4_height, L"");

	p.setfont(font, 24);
	p.set_text_pos(50, 700);
	p.show(L"Hello, world/signed!");
	p.continue_text(L"(says C)");
	p.end_page_ext(L"");

	p.end_document(L"");

	/* fetch the buffer containing the PDF document */
	contents = p.get_buffer(&size);
    }
    catch (PDFlib::Exception &ex) {
        wcerr << L"PDFlib exception occurred in hellosign sample:" << endl;
        wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
	    << L": " << ex.get_errmsg() << endl;
        
        return(98);
    }

    /* ------------------ Step 2: sign the document ----------------- */
	
    try {
	int doc;
	PLOP plop;
	wstring optlist = L"searchpath={" + searchpath + L"}";

	plop.set_option(optlist);

	/* Create a virtual PVF file from the data in memory */
	plop.create_pvf(L"/pvf/hello.pdf", contents, size, L"");

	/* open unsigned input document from PVF file */
	if ((doc = plop.open_document(L"/pvf/hello.pdf", L"")) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return(99);
	}

	/* sign the output file */
	if (plop.create_file(out_filename, sign_opts) == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return(99);
	}

        /* close the output file */
        plop.close_document(doc);

        /* delete PVF file containing input document */
        if (plop.delete_pvf(L"/pvf/hello.pdf") == -1)
	{
	    wcerr << L"Error: " << plop.get_errmsg() << endl;
	    return(99);
	}

    }
    catch (PLOP::Exception &ex) {
	wcerr << L"PLOP exception occurred in hellosign:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
		<< L": " << ex.get_errmsg() << endl;
	return(99);
    }

    return 0;
}
