/* $Id: LinearizeServlet.java,v 1.1 2011/03/08 11:51:15 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * linearization sample
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

import com.pdflib.PLOPException;
import com.pdflib.plop;

public class LinearizeServlet extends HttpServlet {
    private static final long serialVersionUID = -807008837302936073L;

    public void service(HttpServletRequest request, HttpServletResponse response)
            throws ServletException {
        plop plop = null;

        try {
            final String in_filename = "PLOP-datasheet.pdf";
            final String out_filename = "";

            plop = new plop();

            /*
             * Set up searchpath for the required data.
             */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            /*
             * Catch problems that application data cannot be accessed.
             */
            if (searchpath == null) {
                throw new ServletException(
                        "Error: Unable to access application data (WAR file not unpacked?)");
            }
            
            plop.set_option("searchpath={" + searchpath + "}");

            /* open input file */
            int doc;
            if ((doc = plop.open_document(in_filename, "")) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            /* create the linearized output file */
            if (plop.create_file(out_filename,
                    "linearize docinfo={Author {PDFlib GmbH}}") == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            final byte[] contents = plop.get_buffer();

            response.setContentType("application/pdf");
            response.setContentLength(contents.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(contents);
            out.close();

            /* close input and output files */
            plop.close_document(doc);
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();
        }
    }
}
