/* $Id: NoPrintServlet.java,v 1.1 2011/03/08 11:51:15 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * sample for setting permissions
 */

import javax.servlet.http.HttpServlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import com.pdflib.PLOPException;
import com.pdflib.plop;

public class NoPrintServlet extends HttpServlet {
    private static final long serialVersionUID = -670993099271431783L;

    public void service(ServletRequest request, ServletResponse response)
            throws ServletException {
        plop plop = null;

        try {
            /* parameters for the input document */
            final String in_filename = "PLOP-datasheet.pdf";
            final String in_password = "";

            /* parameters for the output document */
            final String out_filename = "";
            final String out_master = "DEMO";
            final String out_user = "";
            final String permissions = "noprint nocopy";

            plop = new plop();

            /*
             * Set up searchpath for the required data.
             */
            final ServletContext context = getServletContext();
            final String searchpath = context.getRealPath("/WEB-INF/data");

            /*
             * Catch problems that application data cannot be accessed.
             */
            if (searchpath == null) {
                throw new ServletException(
                        "Error: Unable to access application data (WAR file not unpacked?)");
            }
            
            plop.set_option("searchpath={" + searchpath + "}");
            
            /* open unprotected input file without any password */
            String optlist = "password {" + in_password + "} ";
            int doc;
            if ((doc = plop.open_document(in_filename, optlist)) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            /* create the output file with a password */
            optlist = "masterpassword {" + out_master + "} userpassword {"
                    + out_user + "} permissions {" + permissions + "}";
            if (plop.create_file(out_filename, optlist) == -1) {
                throw new Exception("Error: " + plop.get_errmsg());
            }

            final byte[] buf = plop.get_buffer();

            response.setContentType("application/pdf");
            response.setContentLength(buf.length);

            final ServletOutputStream out = response.getOutputStream();
            out.write(buf);
            out.close();

            /* close input and output files */
            plop.close_document(doc);
        }
        catch (PLOPException e) {
            throw new ServletException("PLOP exception occurred in "
                    + getClass().getName(), e);
        }
        catch (Exception e) {
            throw new ServletException("Exception occurred in "
                    + getClass().getName(), e);
        }
        finally {
            /* delete the PLOP object */
            if (plop != null)
                plop.delete();
        }
    }
}
