/* $Id: sign.java,v 1.12 2011/03/08 11:11:43 stm Exp $
 *
 * PDFlib PLOP: PDF Linearization, Optimization, Protection
 * sample for signing a document
 */

import java.io.*;
import com.pdflib.plop;
import com.pdflib.PLOPException;

public class sign
{
    public static void main (String argv[])
    {
	plop plop = null;

	try {
	    /* parameters for the input document */
	    String optlist;
	    String searchpath = "../data";
	    
	    /*
	     * alternatives for specifying the certificate
	     * 
	     * Windows Certificate Store (Windows only):
             *  "engine=mscapi" +
	     *  "digitalid={certstore={store=My subject={DEMO PLOP user 2048}}}"
	     * PFX certificate from file in search path:
	     *    "engine=builtin " +
	     *    "digitalid={filename=demo2048.pfx} "
             * PKCS#12 certificate from file in search path:
	     *    "engine=builtin " +
	     *    "digitalid={filename=demo2048.p12} "
             * PKCS#11 certificate
	     *    "engine=pkcs#11 " +
	     *    "digitalid={filename=cryptoki.dll} "
	     */
	    String sign_opts = "sign={"  +
                "appearance={fieldname=Signature1 } " +
	        "engine=builtin " +
                "digitalid={filename=demo2048.p12} " +
                "password={demo} " +
                "reason={I agree to this document} " +
                "contactinfo={tel (123) 456 789} " +
                "location=Munich " +
            "}";
	    int doc;


	    if (argv.length != 2)
	    {
		throw new Exception("usage: sign  <filename> <outfilename>");
	    }

	    plop = new plop();

	    /* open unprotected input file without any password */
	    optlist = "searchpath {" + searchpath + "} ";
	    plop.set_option(optlist);
	    if ((doc = plop.open_document(argv[0], "")) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }

	    /* create the output file with a password */
	    if (plop.create_file(argv[1], sign_opts) == -1) {
		throw new Exception("Error: " + plop.get_errmsg());
	    }

	    /* close input and output files */
	    plop.close_document(doc);

        } catch (PLOPException e) {
            System.err.print("PLOP exception occurred in sign sample:\n");
            System.err.print("[" + e.get_errnum() + "] " + e.get_apiname() +
                            ": " + e.get_errmsg() + "\n");
        } catch (Exception e) {
            System.err.println(e.getMessage());
	} finally {
	    /* delete the PLOP object */
	    if (plop != null) plop.delete();
	}
    }
}
