#!/usr/bin/perl
# $Id: dumper.pl,v 1.9 2011/02/21 13:06:16 rjs Exp $
#

use PDFlib::PLOP 4.1;

my $doc;

# This is where input files live. Adjust as necessary.
my $searchpath = "../data";

# create a new PLOP object
my $plop = new PDFlib::PLOP;


if ($#ARGV != 0) {
    die("usage: dumper <filename>\n");
}

eval {
    my ($count, $pcosmode, $plainmetadata);
    my ($objtype, $i, $doc);
    my $docoptlist = "requiredmode=minimum";
    my $globaloptlist = "";
    my $optlist;

    $plop->set_option($globaloptlist);

    $optlist = sprintf("searchpath={%s}", $searchpath);
    $plop->set_option($optlist);

    if (($doc = $plop->open_document($ARGV[0], $docoptlist)) == -1)
    {
	printf("ERROR: %s\n", $plop->get_errmsg());
	$plop->delete();
	exit;
    }

    # --------- general information (always available)

    $pcosmode = $plop->pcos_get_number($doc, "pcosmode");

    printf("   File name: %s\n", $plop->pcos_get_string($doc, "filename"));

    printf(" PDF version: %s\n",
	$plop->pcos_get_string($doc, "pdfversionstring"));

    printf("  Encryption: %s\n",
	$plop->pcos_get_string($doc, "encrypt/description"));

    printf("   Master pw: %s\n",
	$plop->pcos_get_number($doc, "encrypt/master") ? "yes" : "no");

    printf("     User pw: %s\n",
	$plop->pcos_get_number($doc, "encrypt/user") ? "yes" : "no");

    printf("Text copying: %s\n",
	$plop->pcos_get_number($doc, "encrypt/nocopy") ? "no" : "yes");

    printf("  Linearized: %s\n",
	$plop->pcos_get_number($doc, "linearized") ? "yes" : "no");

    if ($pcosmode == 0) {
	printf("Minimum mode: no more information available\n\n");
	$plop->close_document($doc);
	$plop->delete();
	exit;
    }

    # --------- more details (requires at least user password)

    printf("PDF/X status: %s\n", $plop->pcos_get_string($doc, "pdfx"));

    printf("PDF/A status: %s\n", $plop->pcos_get_string($doc, "pdfa"));

    printf("    XFA data: %s\n",
        $plop->pcos_get_string($doc, "type:/Root/AcroForm/XFA") ne "null"
                            ? "yes" : "no");

    printf("  Tagged PDF: %s\n\n",
	$plop->pcos_get_number($doc, "tagged") ? "yes" : "no");

    printf("No. of pages: %d\n",
	$plop->pcos_get_number($doc, "length:pages"));

    printf(" Page 1 size: width=%g, height=%g\n",
	$plop->pcos_get_number($doc, "pages[0]/width"),
	$plop->pcos_get_number($doc, "pages[0]/height"));

    $count = $plop->pcos_get_number($doc, "length:fonts");
    printf("No. of fonts: %d\n", $count);

    for ($i=0; $i < $count; $i++) {
	if ($plop->pcos_get_number($doc, "fonts[$i]/embedded")) {
	    printf("embedded ");
	}
	else {
	    printf("unembedded ");
	}

	printf("%s font ",
	    $plop->pcos_get_string($doc, "fonts[$i]/type"));
	printf("%s\n",
	    $plop->pcos_get_string($doc, "fonts[$i]/name"));
    }

    printf("\n");

    $plainmetadata =
	$plop->pcos_get_number($doc, "encrypt/plainmetadata");

    if ($pcosmode == 1 && !$plainmetadata &&
	( $plop->pcos_get_number($doc, "encrypt/nocopy"))) {
	printf("Restricted mode: no more information available\n\n");
	$plop->close_document($doc);
	$plop->delete();
	exit;
    }

    # --------- document info keys and XMP metadata (requires master pw
    # or plaintext metadata)
    #

    $count = $plop->pcos_get_number($doc, "length:/Info");

    for ($i=0; $i < $count; $i++) {
	$objtype = $plop->pcos_get_string($doc, "type:/Info[$i]");
	printf("%12s: ", $plop->pcos_get_string($doc, "/Info[$i].key"));

	# Info entries can be stored as string or name objects
	if ($objtype eq "string" || $objtype eq "name")
	{
	    printf("'%10s'\n",
		    $plop->pcos_get_string($doc, "/Info[$i]"));
	}
	else
	{
	    printf("(%s object)\n",
		$plop->pcos_get_string($doc, "type:/Info[$i]"));
	}
    }

    printf("\nXMP meta data: ");

    $objtype = $plop->pcos_get_string($doc, "type:/Root/Metadata");
    if ($objtype eq "stream") {
	my $contents;

	$contents = $plop->pcos_get_stream($doc, "", "/Root/Metadata");
	printf("%d bytes ", length($contents));

	# This demonstrates Unicode conversion 
	$contents = $plop->convert_to_unicode("utf8", $contents,
						"outputformat=utf16");
	printf("(%d Unicode characters)\n\n", length($contents)/2);
    }
    else {
	printf("not present\n\n");
    }

    $plop->close_document($doc);
};
if ($@) {
    die("$0: PLOP Exception occurred:\n$@");
}
