# PLOP sample application for dumping PDF information with PLOP
#
# $Id: dumper.py,v 1.7 2011/02/24 10:03:15 rjs Exp $
#

from sys import *
from traceback import *
from plop_py import *

def yesno(arg):
    if (arg != 0):
        return "yes"
    return "no"


if len(argv) != 2:
    raise Exception("usage: dumper <filename>")

try:
    try:
        plop = PLOP_new()

        docoptlist = "requiredmode=minimum"
        globaloptlist = ""
        searchpath = "../data"

        PLOP_set_option(plop, globaloptlist)
        PLOP_set_option(plop, "searchpath={" + searchpath + "}")

        doc = PLOP_open_document(plop, argv[1], docoptlist)

        if doc  == -1:
            raise Exception("ERROR: %s\n" % PLOP_get_errmsg(plop))

        # --------- general information (always available) */
        pcosmode = int(PLOP_pcos_get_number(plop, doc, "pcosmode"))

        print(("   File name: %s" % \
        PLOP_pcos_get_string(plop, doc, "filename")))

        print((" PDF version: %s" % \
            PLOP_pcos_get_string(plop, doc, "pdfversionstring")))

        print(("  Encryption: %s" % \
            PLOP_pcos_get_string(plop, doc, "encrypt/description")))

        print(("   Master pw: %s" % \
            yesno(PLOP_pcos_get_number(plop, doc, "encrypt/master"))))

        print(("     User pw: %s" % \
            yesno(PLOP_pcos_get_number(plop, doc, "encrypt/user"))))

        print(("Text copying: %s" % \
            yesno(not PLOP_pcos_get_number(plop, doc, "encrypt/nocopy"))))

        print(("  Linearized: %s" % \
            yesno(PLOP_pcos_get_number(plop, doc, "linearized"))))

        if (pcosmode == 0):
            print("Minimum mode: no more information available\n")
            PLOP_close_document(plop, doc)
            exit(0)

        # --------- more details (requires at least user password) */

        print(("PDF/X status: %s" % PLOP_pcos_get_string(plop, doc, "pdfx")))

        print(("PDF/A status: %s" % PLOP_pcos_get_string(plop, doc, "pdfa")))

        print("    XFA data: %s" % \
            yesno(PLOP_pcos_get_number(plop, doc, "type:/Root/AcroForm/XFA") != 0))

        print(("  Tagged PDF: %s\n" % \
            yesno(PLOP_pcos_get_number(plop, doc, "tagged"))))

        print(("No. of pages: %d" % \
            int(PLOP_pcos_get_number(plop, doc, "length:pages"))))

        print((" Page 1 size: width=%g, height=%g" % \
            (PLOP_pcos_get_number(plop, doc, "pages[%d]/width" % 0),
             PLOP_pcos_get_number(plop, doc, "pages[%d]/height" % 0))))

        count = int(PLOP_pcos_get_number(plop, doc, "length:fonts"))
        print(("No. of fonts: %d" % count))

        for i in range(count):
            type = PLOP_pcos_get_string(plop, doc, "fonts[%d]/type" % i)
            name = PLOP_pcos_get_string(plop, doc, "fonts[%d]/name" % i)

            if (PLOP_pcos_get_number(plop, doc, "fonts[%d]/embedded" % i)):
                print("embedded %s font %s" % (type, name))
            else:
                print("unembedded %s font %s" % (type, name))

        print

        plainmetadata = \
                int(PLOP_pcos_get_number(plop, doc, "encrypt/plainmetadata"))

        if (pcosmode == 1 and not plainmetadata and \
                int(PLOP_pcos_get_number(plop, doc, "encrypt/nocopy"))):
            print("Restricted mode: no more information available\n")
            PLOP_close_document(plop, doc)
            exit(0)

        # --------- document info keys and XMP metadata (requires master pw
        # or plaintext metadata)
        #

        count = int(PLOP_pcos_get_number(plop, doc, "length:/Info"))

        for i in range(0, count):
            objtype = int(PLOP_pcos_get_number(plop, doc, "type:/Info[%d]" % i))
            key = PLOP_pcos_get_string(plop,doc, "/Info[%d].key" % i)

            # Info entries can be stored as string or name objects */
            # plop_to_sting == 4; plop_ot_name == 3
            if (objtype == 4 or objtype == 3):
                print("%12s: '%10s'" % \
                    (key, PLOP_pcos_get_string(plop, doc, "/Info[%d]" % i)))
            else:
                print("%12s: (%s object)" % \
                   (key, PLOP_pcos_get_string(plop, doc, "type:/Info[%d]" % i)))

        print

        objtype = int(PLOP_pcos_get_number(plop, doc, "type:/Root/Metadata"))
        # plop_ot_stream == 7
        if (objtype == 7):
            contents = PLOP_pcos_get_stream(plop, doc, "", "/Root/Metadata")
            ustring = PLOP_convert_to_unicode(plop, "utf8", contents, "outputformat=utf16")
            print("XMP meta data: %d bytes (%d Unicode characters)\n" % 
                (len(contents), len(ustring)/2))
        else:
            print("XMP meta data: not present\n")

        PLOP_close_document(plop, doc)

    except PLOPException:
        print_exc(file=stdout)

except Exception:
    print_exc(file=stdout)

finally:
    PLOP_delete(plop)
